# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os

from itertools import product
from subprocess import PIPE

from law.util import interruptable_popen, colored

from .FnloCppreadFinal import FnloCppreadFinal
from .Recombine import Recombine

from analysis.framework import Task, SpaceSeparatedListParameter

logger = logging.getLogger(__name__)


class Absolute(Task, law.LocalWorkflow):
    command = 'fastnnlo_absolute.py'

    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only")
    )
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'channel': ch, 'observable': obs}
            for i, (ch, obs) in enumerate(product(
                self.channels, self.observables
            ))
        }

    def run_configs(self):
        """Run configuration dicts. `run` method loops through these to populate the output directory."""
        return [
            {'ylim2_arg': ylim2_arg}
            for ylim2_arg in (None, '0.02')
        ]

    def workflow_requires(self):
        return {'fnlocppread': FnloCppreadFinal(), 'recombine': Recombine()}

    def output(self):
        return law.LocalDirectoryTarget(
            '{}/{}/{}/{}/{}'.format(
                self.plots_dir, self.name, self.__class__.__name__,
                self.branch_data['channel'],
                self.branch_data['observable']
            )
        )

    def run(self):
        datfile = '{}/{}/Combined/Final/{}.{}.{}.dat'.format(
            self.merge_dir, self.name, self.process,
            self.branch_data['channel'],
            self.branch_data['observable']
        )

        try:
            with self.output().temporary_path() as self.temp_output_path:
                os.mkdir(self.temp_output_path)

                outfile = '{}/{}.{}.{}'.format(
                    self.temp_output_path, self.process,
                    self.branch_data['channel'],
                    self.branch_data['observable']
                )

                for i, run_config in enumerate(self.run_configs()):
                    additional_args = (
                        ['--ylim2', run_config['ylim2_arg']]
                        if run_config['ylim2_arg'] is not None
                        else []
                    )

                    command_line_args = [
                        self.command, '-f', outfile
                    ] + additional_args + [datfile]

                    logger.info("Running command [%s/%s]: %s", i+1,
                        len(self.run_configs()),
                        colored(' '.join(command_line_args), 'yellow'))
                    code, out, error = interruptable_popen(command_line_args,
                                                           stdout=PIPE,
                                                           stderr=PIPE)

                    logger.debug("Return code: %d", code)
                    logger.debug("Standard output:\n%s\n", out)
                    logger.debug("Standard error:\n%s\n", error)

                if (code == 0):
                    logger.info("%s status: SUCCESS", self.command)
                else:
                    raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            os.rmdir(self.temp_output_path)
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
