# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re

from functools import partial

from .Rewarmup import Rewarmup
from ..util import working_directory

from analysis.framework import Task, SpaceSeparatedListParameter

logger = logging.getLogger(__name__)
working_directory = partial(working_directory, logger=logger)


class CopyRewarmup(Task, law.LocalWorkflow):
    force = luigi.BoolParameter(default=False)

    warm_dir = luigi.Parameter(default='warmups')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RRa", "RRb", "RV", "VV")
    )

    jobnameext = luigi.Parameter()
    cutoff = luigi.Parameter()
    kinetags = luigi.DictParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        return {
            i : {'index': i, 'channel': channel,
                 'kinetag': self.kinetags[channel]}
            for i, channel in enumerate(self.channels)
        }

    def workflow_requires(self):
        if self.force:
            return {"rewarmup": Rewarmup.req(self, acceptance=0)}
        else:
            return {"rewarmup": Rewarmup.req(self)}

    def requires(self):
        return {'rewarmup': Rewarmup(branch=self.branch_data['index'])}

    def output(self):
        return law.LocalFileTarget(
            '{}/{}/{}/{}.{}-{}.{}.{}'.format(
                self.warm_dir, 'rewarmup', self.name,
                self.process, self.branch_data['channel'],
                self.jobnameext, self.cutoff, self.branch_data['kinetag']
            )
        )

    def run(self):
        self.output().parent.touch()
        with working_directory(self.output().parent.path):
            inp = Rewarmup().output()['collection'].targets[self.branch]
            if not inp.exists():
                message = 'Rewarmup result not found: {} (branch {})'.format(inp, self.branch)
                logger.error(message)
                raise RuntimeError(message)
            else:
                logger.info('Copying Rewarmup result: %s -> %s/%s/%s',
                    inp, self.warm_dir, 'rewarmup', self.name
                )
                inp.load('')
