# -*- coding: utf-8 -*-
import glob
import law
import luigi
import logging
import os
import re
import shutil

from itertools import product
from subprocess import PIPE

from law.util import interruptable_popen, colored

from ..util import working_directory

from .AddStatunc import AddStatunc

from analysis.framework import Task

logger = logging.getLogger(__name__)

class DataComparison(Task, law.LocalWorkflow):
    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    orders = luigi.ListParameter()
    observables = luigi.ListParameter()
    rivetid = luigi.Parameter()

    pdf = luigi.Parameter()
    scalenames = luigi.ListParameter()
    uncertainty = luigi.Parameter(default='6P')
    xrescale = luigi.Parameter(default='1')
    dxsrescale = luigi.Parameter(default='1')

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'scalename': sclnam}
            for i, sclnam in enumerate(self.scalenames)
        }

    def workflow_requires(self):
        return {'addstatunc': AddStatunc(self)}

    def requires(self):
        return AddStatunc()

    def output(self):
        return law.LocalDirectoryTarget(
            '{}/{}/DataComparison/{}'.format(
                self.plots_dir, self.name, self.branch_data['scalename']
            )
        )

    def run(self):
        try:
            rundirpath = os.getcwd()

            tabpath = '{}/{}/Combined/Final'.format(self.merge_dir, self.name)
            tabext = '_v25.tab.gz'
            tabunc = self.uncertainty

            yodanames = {}
            yodafiles = {}

            # temporarily change to dir with final grids and dat files
            with working_directory(tabpath):

                # Initial remove of potentially existing yoda files to collect order-wise results for all observables
                for order in self.orders:
                    yodanames[order] = '{}.{}.{}.{}.{}.{}.yoda'.format(
                        self.process, self.name, order, self.pdf, tabunc, self.branch_data['scalename']
                    )
                    if os.path.isfile(yodanames[order]):
                        logger.debug('Removing existing YODA file for order %s: %s',
                            order, yodanames[order]
                        )
                        os.remove(yodanames[order])

                # -- subtask 1: run `fnlo-tk-yodaout` to produce YODA files to
                #               be read in by Rivet

                # loop over all observables and orders
                for (order, obs) in product(self.orders, self.observables):

                    # Define tab path and name
                    tabfile = None
                    for taborder in self.orders:
                        if self.orders.index(taborder) < self.orders.index(order):
                            continue
                        tabbase = '{}.{}.{}'.format(self.process, taborder, obs)
                        tabtest = tabbase + tabext
                        if not os.path.exists(tabtest):
                            continue
                        tabfile = tabtest
                        break
                    if not tabfile:
                        raise FileNotFoundError('Could not find grid usable for order %s.', order)
                    logfile = tabbase + '_yodaout_' + tabunc + '_' + self.branch_data['scalename'] + '.log'

                    # Execute fnlo-tk-yodaout
                    command_line_args = [
                        'fnlo-tk-yodaout', tabfile, self.pdf, tabunc, order, '_',
                        self.branch_data['scalename'], '_', '_', self.xrescale, self.dxsrescale
                    ]
                    logger.info("Running command: %s",
                        colored(' '.join(command_line_args), 'yellow'))

                    code, out, error = interruptable_popen(command_line_args, stdout=PIPE, stderr=PIPE)

                    logger.debug("Return code: %d", code)
                    logger.debug("Standard output:\n%s\n", out)
                    logger.debug("Standard error:\n%s\n", error)

                    if (code == 0):
                        logger.info("fnlo-tk-yodaout status: SUCCESS")
                    else:
                        raise RuntimeError("fnlo-tk-yodaout status: FAIL (exit code {})".format(code))

                    logger.debug("Dumping standard out to file:  %s", logfile)
                    with open(logfile, 'w') as f:
                        f.write(out)

                    # concatenate YODA files for each order
                    yinfiles = glob.glob('{}_*_{}_*.yoda'.format(order,self.branch_data['scalename']))
                    with open(yodanames[order], 'a') as youtfile:
                        for fname in yinfiles:
                            with open(fname, 'r') as yin:
                                youtfile.write(yin.read())
                            os.remove(fname)

                # -- subtask 2: execute Rivet to produce data comparison plots
                #               for all grids (= histograms)
                command_line_args = [
                    'rivet-mkhtml',
                    '--mc-errs', '-o{}'.format(self.branch_data['scalename'])
                ]
                legadd = tabunc
                #                if self.dxsrescale != 1:
                #                    legadd = tabunc + '*' + str(self.dxsrescale)
                for order in self.orders:
                    if order == 'LO':
                        command_line_args.append(yodanames[order] + ":'NNLOJET+-" + legadd + " LO':LineColor=green")
                    elif order == 'NLO':
                        command_line_args.append(yodanames[order] + ":'NLO':LineColor=blue")
                    elif order == 'NNLO':
                        if tabunc not in ["2P", "6P"]:
                            command_line_args.append(yodanames[order] + ":'NNLO':LineColor=red")
                        else:
                            command_line_args.append(yodanames[order] + ":'NNLO':LineColor=red:ErrorBands=1:ErrorBandColor=orange")

                logger.info("Running command: %s",colored(' '.join(command_line_args), 'yellow'))
                code, out, error = interruptable_popen(command_line_args, stdout=PIPE, stderr=PIPE)

                logger.debug("Return code: %d", code)
                logger.debug("Standard output:\n%s\n", out)
                logger.debug("Standard error:\n%s\n", error)

                if (code == 0):
                    logger.info("rivet-mkhtml status: SUCCESS")
                else:
                    raise RuntimeError("rivet-mkhtml status: FAIL (exit code {})".format(code))

                # -- subtask 3: create png plots, if missing for annoying security
                #               issues with ImageMagick convert used by Rivet

                # temporarily change to subdirectory created by `rivet-mkhtml`
                subdir = self.branch_data['scalename'] + "/" + self.rivetid
                with working_directory(subdir):
                    pdffiles = glob.glob('*.pdf')
                    for pdffile in pdffiles:
                        outname = re.sub(r'\.pdf$', '', pdffile)
                        outfile = outname + '.png'
                        if not os.path.exists(outfile):
                            command_line_args = [
                                'pdftoppm', '-png', '-singlefile',
                                '-r', '150', pdffile, outname
                            ]

                            logger.info("Running command: %s",
                                colored(' '.join(command_line_args), 'yellow'))
                            code, out, error = interruptable_popen(command_line_args, stdout=PIPE, stderr=PIPE)

                            logger.debug("Return code: %d", code)
                            logger.debug("Standard output:\n%s\n", out)
                            logger.debug("Standard error:\n%s\n", error)

                            if (code != 0):
                                logger.warning('Could not produce missing png plot: %s'.format(outfile))

                # Move produced folder with Rivet web pages and plots to destination in plots folder
                dstdir = os.path.join(
                    rundirpath, self.plots_dir, self.name, 'DataComparison'
                )
                if not os.path.isdir(dstdir):
                    os.mkdir(dstdir)

                logger.debug('Moving `rivet-mkhtml` output: %s -> %s', os.getcwd(), dstdir)
                shutil.move(self.branch_data['scalename'], dstdir)

        except Exception as e:
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
