# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os

from itertools import product
from subprocess import PIPE

from law.util import interruptable_popen, colored

from .FnloCppreadFinal import FnloCppreadFinal
from .Recombine import Recombine

from analysis.framework import Task, SpaceSeparatedListParameter

logger = logging.getLogger(__name__)


class MergedGridClosure(Task, law.LocalWorkflow):
    command = 'fastnnlo_scaleclosure.py'

    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    allchannels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only"]
    channels = SpaceSeparatedListParameter(
        default=allchannels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only")
    )
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'channel': ch, 'observable': obs}
            for i, (ch, obs) in enumerate(product(
                self.channels,
                self.observables,
            ))
        }

    def workflow_requires(self):
        return {'fnlocppread': FnloCppreadFinal(), 'recombine': Recombine()}

    def output(self):
        return law.LocalDirectoryTarget(
            '{}/{}/{}/{}/{}'.format(
                self.plots_dir, self.name, self.__class__.__name__,
                self.branch_data['channel'],
                self.branch_data['observable']
            )
        )

    def run(self):
        datfile = '{}/{}/Combined/Final/{}.{}.{}.dat'.format(
            self.merge_dir, self.name, self.process,
            self.branch_data['channel'],
            self.branch_data['observable']
        )

        logfile = '{}/{}/Combined/Final/{}.{}.{}.log'.format(
            self.merge_dir, self.name, self.process,
            self.branch_data['channel'],
            self.branch_data['observable']
        )

        try:
            with self.output().temporary_path() as self.temp_output_path:
                os.mkdir(self.temp_output_path)

                outfile = '{}/{}.{}.{}'.format(
                    self.temp_output_path, self.process,
                    self.branch_data['channel'],
                    self.branch_data['observable']
                )

                command_line_args = [
                    self.command,
                    '-d', datfile, '-l', logfile,
                    '-f', outfile
                ]

                logger.info("Running command: %s",
                    colored(' '.join(command_line_args), 'yellow'))
                code, out, error = interruptable_popen(command_line_args,
                                                       stdout=PIPE,
                                                       stderr=PIPE)

                logger.debug("Return code: %d", code)
                logger.debug("Standard output:\n%s\n", out)
                logger.debug("Standard error:\n%s\n", error)

                if (code == 0):
                    logger.info("%s status: SUCCESS", self.command)
                else:
                    raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            os.rmdir(self.temp_output_path)
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
